package org.processmining.blocklayout.algorithms;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.processmining.blocklayout.models.reductions.Reduction;
import org.processmining.blocklayout.models.reductions.TransitionReduction;
import org.processmining.blocklayout.models.statemachine.StateMachineNet;
import org.processmining.blocklayout.models.statemachine.Transition;
import org.processmining.blocklayout.parameters.LayoutParameters;

public class AlternativeTransitionFinderAlgorithm implements FinderAlgorithm {

	/*
	 * Returns transition reductions where the transitions have identical pre- and
	 * postset. Note that the net is a state machine, hence every transition has
	 * a single input place and a single output place.
	 */
	public Set<Reduction> find(StateMachineNet net, LayoutParameters layoutParameters) {
		Set<Reduction> reductions = new HashSet<Reduction>();
		for (Transition transition : net.getTransitionList()) {
			for (Transition alternativeTransition : net.getPostset(net.getPrePlace(transition))) {
				if (transition != alternativeTransition) {
					if (net.getPostPlace(alternativeTransition) == net.getPostPlace(transition)) {
						List<Transition> transitions = new ArrayList<Transition>();
						transitions.add(alternativeTransition);
						Reduction reduction = new TransitionReduction(transition, transitions, false);
						reductions.add(reduction);
						/*
						 * Found a reduction. As this reduction has costs 0, it can be applied immediately.
						 */
						return reductions;
					}
				}
			}
		}
		return reductions;
	}

}
