package org.processmining.blocklayout.algorithms;

import java.util.HashSet;
import java.util.Set;

import org.processmining.blocklayout.models.reductions.PlaceReduction;
import org.processmining.blocklayout.models.reductions.Reduction;
import org.processmining.blocklayout.models.statemachine.Place;
import org.processmining.blocklayout.models.statemachine.StateMachineNet;
import org.processmining.blocklayout.parameters.LayoutParameters;

public class PlaceFinderAlgorithm implements FinderAlgorithm {

	private boolean isReversed;

	public PlaceFinderAlgorithm(boolean isReversed) {
		this.isReversed = isReversed;
	}
	
	public Set<Reduction> find(StateMachineNet net, LayoutParameters layoutParameters) {
		int bestCosts = -1;
		Set<Reduction> reductions = new HashSet<Reduction>();
		nextPlace: for (Place place : net.getPlaceList()) {
			for (Place proxy : net.getPlaceList()) {
				if (place == proxy) {
					continue nextPlace;
				}
				Reduction reduction = new PlaceReduction(place, proxy, isReversed);
				int costs = reduction.costs(net, layoutParameters);
				if (costs == 0) {
					reductions.clear();
					reductions.add(reduction);
					return reductions;
				} else if (costs < bestCosts || bestCosts == -1) {
					bestCosts = costs;
					reductions.clear();
					reductions.add(reduction);
				}
			}
		}
		return reductions;
	}

}
