package org.processmining.blocklayout.algorithms;

import java.util.HashSet;
import java.util.Set;

import org.processmining.blocklayout.models.reductions.Reduction;
import org.processmining.blocklayout.models.reductions.RigidReduction;
import org.processmining.blocklayout.models.statemachine.StateMachineNet;
import org.processmining.blocklayout.models.statemachine.Transition;
import org.processmining.blocklayout.parameters.LayoutParameters;

public class RigidFinderAlgorithm implements FinderAlgorithm {

	/**
	 * Find a rigid.
	 */
	public Set<Reduction> find(StateMachineNet net, LayoutParameters layoutParameters) {
		Set<Reduction> reductions = new HashSet<Reduction>();
		for (Transition transition : net.getTransitionList()) {
			System.out.println("[RigidFinderAlgorithm] Transition " + transition);
			if (net.getPrePlace(transition).equals(net.getPostPlace(transition))) {
				continue;
			}
			Transition splitTransition = getSplitTransition(net, transition);
			if (splitTransition != null) {
				System.out.println("[RigidFinderAlgorithm] Split place " + splitTransition);
				Transition joinTransition = getJoinTransition(net, transition);
				if (joinTransition != null) {
					System.out.println("[RigidFinderAlgorithm] Join place " + joinTransition);
					reductions.add(new RigidReduction(splitTransition, transition, joinTransition));
					return reductions;
				}
			}
		}
		return reductions;
	}

	private Transition getSplitTransition(StateMachineNet net, Transition transition) {
		if (net.getPreset(net.getPrePlace(transition)).size() != 1) {
			return null;
		}
		if (net.getPreset(net.getPostPlace(transition)).size() != 2) {
			return null;
		}
		for (Transition transition1 : net.getPreset(net.getPrePlace(transition))) {
			if (transition1.equals(transition)) {
				continue;
			}
			for (Transition transition2 : net.getPreset(net.getPostPlace(transition))) {
				if (transition2.equals(transition)) {
					continue;
				}
				if (transition2.equals(transition1)) {
					continue;
				}
				if (net.getPrePlace(transition1).equals(net.getPrePlace(transition2))) {
					System.out.println("[RigidFinderAlgorithm] transitions " + transition1 + ", " + transition2);
					return transition1;
				}
			}
		}
		return null;
	}

	private Transition getJoinTransition(StateMachineNet net, Transition transition) {
		if (net.getPostset(net.getPostPlace(transition)).size() != 1) {
			return null;
		}
		if (net.getPostset(net.getPrePlace(transition)).size() != 2) {
			return null;
		}
		for (Transition transition1 : net.getPostset(net.getPostPlace(transition))) {
			if (transition1.equals(transition)) {
				continue;
			}
			for (Transition transition2 : net.getPostset(net.getPrePlace(transition))) {
				if (transition2.equals(transition)) {
					continue;
				}
				if (transition2.equals(transition1)) {
					continue;
				}
				if (net.getPostPlace(transition1).equals(net.getPostPlace(transition2))) {
					System.out.println("[RigidFinderAlgorithm] transitions " + transition1 + ", " + transition2);
					return transition1;
				}
			}
		}
		return null;
	}
}
