package org.processmining.blocklayout.algorithms;

import java.awt.Dimension;
import java.util.HashMap;
import java.util.Map;

import org.processmining.blocklayout.models.blocks.EdgeBlock;
import org.processmining.blocklayout.models.blocks.ElementBlock;
import org.processmining.blocklayout.models.blocks.NodeBlock;
import org.processmining.blocklayout.models.statemachine.Place;
import org.processmining.blocklayout.models.statemachine.StateMachineNet;
import org.processmining.blocklayout.models.statemachine.Transition;
import org.processmining.blocklayout.parameters.LayoutParameters;
import org.processmining.framework.util.Pair;
import org.processmining.models.connections.GraphLayoutConnection;
import org.processmining.models.graphbased.directed.DirectedGraph;
import org.processmining.models.graphbased.directed.DirectedGraphEdge;
import org.processmining.models.graphbased.directed.DirectedGraphElement;
import org.processmining.models.graphbased.directed.DirectedGraphNode;

public class StateMachineBuilderAlgorithm {

	public Pair<StateMachineNet, Map<DirectedGraphElement, ElementBlock>> build(
			DirectedGraph<? extends DirectedGraphNode, ? extends DirectedGraphEdge<? extends DirectedGraphNode, ? extends DirectedGraphNode>> graph,
			GraphLayoutConnection layout, LayoutParameters layoutParameters) {

		/*
		 * Create a net from the graph.
		 */

		/*
		 * Maps every node in the graph onto a place in the net.
		 */
		Map<DirectedGraphNode, Place> nodes = new HashMap<DirectedGraphNode, Place>();

		/*
		 * Maps every element in the graph onto its element block. In the end,
		 * this element block will contain the position of this element.
		 */
		Map<DirectedGraphElement, ElementBlock> blocks = new HashMap<DirectedGraphElement, ElementBlock>();

		/*
		 * Create a net empty net.
		 */
		StateMachineNet net = new StateMachineNet();
		/*
		 * Copy all nodes from the graph as places in the net.
		 */
		for (DirectedGraphNode node : graph.getNodes()) {
			/*
			 * Create a new place for the node.
			 */
			Place place = new Place();
			place.setName(node.getLabel());
			if (place.getName() == null) {
				place.setName(node.toString());
			}
			/*
			 * Get the size for the node.
			 */
			Dimension dimension = layout.getSize(node);
			/*
			 * Create a node block for the place, and set the correct size.
			 */
			double height = layoutParameters.getDefaultHeight();
			double width = layoutParameters.getDefaultWidth();
			if (dimension != null) {
				height = dimension.getHeight();
				width = dimension.getWidth();
			}
			if (!layoutParameters.isLeftRight()) {
				double h = height;
				height = width;
				width = h;
			}
			NodeBlock block = new NodeBlock(place.getName(), height, width, layoutParameters);
			place.setBlock(block);
			/*
			 * Register the place and the block.
			 */
			nodes.put(node, place);
			blocks.put(node, block);
		}
		/*
		 * Copy all edges from the graph as transitions in the net.
		 */
		for (DirectedGraphEdge<? extends DirectedGraphNode, ? extends DirectedGraphNode> edge : graph.getEdges()) {
			/*
			 * Create a new transition.
			 */
			Transition transition = new Transition();
			transition.setName(edge.getSource().getLabel() + edge.getTarget().getLabel());
			/*
			 * Create a new edge block for this transition.
			 */
			EdgeBlock block = new EdgeBlock(transition.getName(), layoutParameters);
			transition.setBlock(block);
			/*
			 * Register the transition and the block.
			 */
			blocks.put(edge, block);

			/*
			 * Add the proper arcs to the net.
			 */
			net.addArc(nodes.get(edge.getSource()), transition);
			net.addArc(transition, nodes.get(edge.getTarget()));
		}
		
		return new Pair<StateMachineNet, Map<DirectedGraphElement, ElementBlock>>(net, blocks);
	}

}
