package org.processmining.blocklayout.dialogs;

import info.clearthought.layout.TableLayout;
import info.clearthought.layout.TableLayoutConstants;

import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JCheckBox;
import javax.swing.JPanel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import org.processmining.blocklayout.parameters.LayoutParameters;
import org.processmining.contexts.uitopia.UIPluginContext;

import com.fluxicon.slickerbox.components.NiceSlider;
import com.fluxicon.slickerbox.components.NiceSlider.Orientation;
import com.fluxicon.slickerbox.factory.SlickerFactory;

public class LayoutDialog extends JPanel {

	/**
	 * 
	 */
	private static final long serialVersionUID = 4422299423330182364L;

	public LayoutDialog(UIPluginContext context, final LayoutParameters layoutParameters) {
		double size[][] = {
				{ TableLayoutConstants.FILL, 50, TableLayoutConstants.FILL },
				{ TableLayoutConstants.FILL, TableLayoutConstants.FILL, TableLayoutConstants.FILL, 20,
						TableLayoutConstants.FILL, TableLayoutConstants.FILL, 20, TableLayoutConstants.FILL } };
		setLayout(new TableLayout(size));
		Dimension preferredDimension = new Dimension(100, 20);
		final double sliderMax = 10000.0;

		final NiceSlider horizontalSpaceSlider = SlickerFactory.instance().createNiceDoubleSlider("Horizontal Space ",
				0.0, 100.0, layoutParameters.getHorizontalSpace(), Orientation.HORIZONTAL);
		horizontalSpaceSlider.addChangeListener(new ChangeListener() {

			public void stateChanged(ChangeEvent e) {
				layoutParameters.setHorizontalSpace(horizontalSpaceSlider.getSlider().getValue() * 100.0 / sliderMax);
			}
		});
		horizontalSpaceSlider.setPreferredSize(preferredDimension);
		add(horizontalSpaceSlider, "0, 0");

		final NiceSlider verticalSpaceSlider = SlickerFactory.instance().createNiceDoubleSlider("Vertical Space ", 0.0,
				100.0, layoutParameters.getVerticalSpace(), Orientation.HORIZONTAL);
		verticalSpaceSlider.addChangeListener(new ChangeListener() {

			public void stateChanged(ChangeEvent e) {
				layoutParameters.setVerticalSpace(verticalSpaceSlider.getSlider().getValue() * 100.0 / sliderMax);
			}
		});
		verticalSpaceSlider.setPreferredSize(preferredDimension);
		add(verticalSpaceSlider, "2, 0");

		final NiceSlider defaultWidthSlider = SlickerFactory.instance().createNiceDoubleSlider("Default Width ", 0.0,
				100.0, layoutParameters.getDefaultWidth(), Orientation.HORIZONTAL);
		defaultWidthSlider.addChangeListener(new ChangeListener() {

			public void stateChanged(ChangeEvent e) {
				layoutParameters.setDefaultWidth(defaultWidthSlider.getSlider().getValue() * 100.0 / sliderMax);
			}
		});
		defaultWidthSlider.setPreferredSize(preferredDimension);
		add(defaultWidthSlider, "0, 1");

		final NiceSlider defaultHeightSlider = SlickerFactory.instance().createNiceDoubleSlider("Default Height ", 0.0,
				100.0, layoutParameters.getDefaultHeight(), Orientation.HORIZONTAL);
		defaultHeightSlider.addChangeListener(new ChangeListener() {

			public void stateChanged(ChangeEvent e) {
				layoutParameters.setDefaultHeight(defaultHeightSlider.getSlider().getValue() * 100.0 / sliderMax);
			}
		});
		defaultHeightSlider.setPreferredSize(preferredDimension);
		add(defaultHeightSlider, "2, 1");

		final NiceSlider startXSlider = SlickerFactory.instance().createNiceDoubleSlider("Start X ", 0.0, 100.0,
				layoutParameters.getStartX(), Orientation.HORIZONTAL);
		startXSlider.addChangeListener(new ChangeListener() {

			public void stateChanged(ChangeEvent e) {
				layoutParameters.setStartX(startXSlider.getSlider().getValue() * 100.0 / sliderMax);
			}
		});
		startXSlider.setPreferredSize(preferredDimension);
		add(startXSlider, "0, 2");

		final NiceSlider startYSlider = SlickerFactory.instance().createNiceDoubleSlider("Start Y ", 0.0, 100.0,
				layoutParameters.getStartY(), Orientation.HORIZONTAL);
		startYSlider.addChangeListener(new ChangeListener() {

			public void stateChanged(ChangeEvent e) {
				layoutParameters.setStartY(startYSlider.getSlider().getValue() * 100.0 / sliderMax);
			}
		});
		startYSlider.setPreferredSize(preferredDimension);
		add(startYSlider, "2, 2");

		final NiceSlider fraction1Slider = SlickerFactory.instance().createNiceDoubleSlider("Fraction 1 ", -1.0, 3.0,
				layoutParameters.getFraction1(), Orientation.HORIZONTAL);
		fraction1Slider.addChangeListener(new ChangeListener() {

			public void stateChanged(ChangeEvent e) {
				layoutParameters.setFraction1((fraction1Slider.getSlider().getValue() / (0.25 * sliderMax)) - 1.0);
			}
		});
		fraction1Slider.setPreferredSize(preferredDimension);
		add(fraction1Slider, "0, 4");

		final NiceSlider fraction2Slider = SlickerFactory.instance().createNiceDoubleSlider("Fraction 2 ", -1.0, 3.0,
				layoutParameters.getFraction2(), Orientation.HORIZONTAL);
		fraction2Slider.addChangeListener(new ChangeListener() {

			public void stateChanged(ChangeEvent e) {
				layoutParameters.setFraction2((fraction2Slider.getSlider().getValue() / (0.25 * sliderMax)) - 1.0);
			}
		});
		fraction2Slider.setPreferredSize(preferredDimension);
		add(fraction2Slider, "2, 4");

		final NiceSlider fraction3Slider = SlickerFactory.instance().createNiceDoubleSlider("Fraction 3 ", -1.0, 3.0,
				layoutParameters.getFraction3(), Orientation.HORIZONTAL);
		fraction3Slider.addChangeListener(new ChangeListener() {

			public void stateChanged(ChangeEvent e) {
				layoutParameters.setFraction3((fraction3Slider.getSlider().getValue() / (0.25 * sliderMax)) - 1.0);
			}
		});
		fraction3Slider.setPreferredSize(preferredDimension);
		add(fraction3Slider, "0, 5");

		final NiceSlider fraction4Slider = SlickerFactory.instance().createNiceDoubleSlider("Fraction 4 ", -1.0, 3.0,
				layoutParameters.getFraction4(), Orientation.HORIZONTAL);
		fraction4Slider.addChangeListener(new ChangeListener() {

			public void stateChanged(ChangeEvent e) {
				layoutParameters.setFraction4((fraction4Slider.getSlider().getValue() / (0.25 * sliderMax)) - 1.0);
			}
		});
		fraction4Slider.setPreferredSize(preferredDimension);
		add(fraction4Slider, "2, 5");

		final JCheckBox leftRightCheckBox = SlickerFactory.instance().createCheckBox(
				"Lay out from top to bottom (instead of from left to right)", false);
		leftRightCheckBox.setSelected(!layoutParameters.isLeftRight());
		leftRightCheckBox.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				layoutParameters.setLeftRight(!leftRightCheckBox.isSelected());
			}

		});
		leftRightCheckBox.setOpaque(false);
		leftRightCheckBox.setPreferredSize(new Dimension(100, 30));
		add(leftRightCheckBox, "0, 7");

		final NiceSlider edgePointThresholdSlider = SlickerFactory.instance().createNiceDoubleSlider(
				"Edge point threshold ", 0.0, 100.0, layoutParameters.getEdgePointThreshold(), Orientation.HORIZONTAL);
		edgePointThresholdSlider.addChangeListener(new ChangeListener() {

			public void stateChanged(ChangeEvent e) {
				layoutParameters
						.setEdgePointThreshold((edgePointThresholdSlider.getSlider().getValue() * 100.0 / sliderMax));
			}
		});
		edgePointThresholdSlider.setPreferredSize(preferredDimension);
		add(edgePointThresholdSlider, "2, 7");

	}
}
