package org.processmining.blocklayout.models.blocks;

import org.processmining.blocklayout.parameters.LayoutParameters;

public interface Block {

	/**
	 * Sets the positions (top-left corners) of all contained element blocks.
	 * 
	 * @param positionX
	 *            The X coordinate of the top-left corner of this block.
	 * @param positionY
	 *            The Y coordinate of the top-left corner of this block.
	 * @param isLeftToRight
	 *            Whether horizontal blocks are to be laid out from left to
	 *            right (or from right to left).
	 * @param layoutParameters
	 *            Parameters.
	 */
	public void setPosition(double positionX, double positionY, boolean isLeftToRight, LayoutParameters layoutParameters);

	/**
	 * Returns the top height of this block. Blocks in a horizontal block will
	 * be laid out such that they align on their 'positionY + topHeight'.
	 * 
	 * @param layoutParameters
	 *            Parameters.
	 * @return The top height of this block.
	 */
	public double getTopHeight(LayoutParameters layoutParameters);

	/**
	 * Returns the bottom height of this block.
	 * 
	 * @param layoutParameters
	 *            Parameters.
	 * @return The bottom height of this block.
	 */
	public double getBottomHeight(LayoutParameters layoutParameters);

	/**
	 * Returns the width of this block.
	 * 
	 * @param layoutParameters
	 *            Parameters
	 * @return The width of this block.
	 */
	public double getWidth(LayoutParameters layoutParameters);
	
	public int getNofLayers();
	
	public String getName();
}
