package org.processmining.blocklayout.models.blocks;

import org.processmining.blocklayout.parameters.LayoutParameters;

public class ElementBlock extends AbstractBlock {

	/**
	 * The position of this element.
	 */
	private double positionX;
	private double positionY;

	/**
	 * The size of this element.
	 */
	private double topHeight;
	private double bottomHeight;
	private double width;

	private String name;
	
	private EdgeBlock edgeBlock1;
	private EdgeBlock edgeBlock2;
	
	/**
	 * Creates an element block with given size. Position is to be set later.
	 * 
	 * @param height
	 *            Height of the block.
	 * @param width
	 *            Width of the block.
	 * @param layoutParameters
	 *            Irrelevant.
	 */
	public ElementBlock(String name, double height, double width, LayoutParameters layoutParameters) {
		/*
		 * Sets top and bottom height to half of the height. If needed, this
		 * causes elements of different heights to be aligned in the middle.
		 */
		setName(name == null ? "<no name>" : name);
		setTopHeight(0.5 * height, layoutParameters);
		setBottomHeight(0.5 * height, layoutParameters);
		setWidth(width, layoutParameters);
		setEdgeBlocks(null, null);
	}

	/**
	 * Sets the position of the top-left corner of this element block.
	 * 
	 * @param positionX
	 *            The X coordinate of the top-left corner.
	 * @param positionX
	 *            The Y coordinate of the top-left corner.
	 * @param isLeftToRight
	 *            Irrelevant.
	 * @param layoutParameters
	 *            Irrelevant.
	 */
	public void setPosition(double positionX, double positionY, boolean isLeftToRight, LayoutParameters layoutParameters) {
		this.setPositionX(positionX);
		this.setPositionY(positionY);
	}

	/*
	 * Getters and setters.
	 */

	public double getTopHeight(LayoutParameters layoutParameters) {
		return topHeight;
	}

	private void setTopHeight(double topHeight, LayoutParameters layoutParameters) {
		this.topHeight = topHeight;
	}

	public double getBottomHeight(LayoutParameters layoutParameters) {
		return bottomHeight;
	}

	private void setBottomHeight(double bottomHeight, LayoutParameters layoutParameters) {
		this.bottomHeight = bottomHeight;
	}

	public double getWidth(LayoutParameters layoutParameters) {
		return width;
	}

	private void setWidth(double width, LayoutParameters layoutParameters) {
		this.width = width;
	}

	public double getPositionX() {
		return positionX;
	}

	private void setPositionX(double positionX) {
		this.positionX = positionX;
	}

	public double getPositionY() {
		return positionY;
	}

	private void setPositionY(double positionY) {
		this.positionY = positionY;
	}
	
	public int getNofLayers() {
		return 1;
	}

	public String getName() {
		return name;
	}

	private void setName(String name) {
		this.name = name;
	}
	
	public String toString() {
		return name;
	}

	public EdgeBlock getEdgeBlock1() {
		return edgeBlock1;
	}

	public EdgeBlock getEdgeBlock2() {
		return edgeBlock2;
	}

	public void setEdgeBlocks(EdgeBlock edgeBlock1, EdgeBlock edgeBlock2) {
		this.edgeBlock1 = edgeBlock1;
		this.edgeBlock2 = edgeBlock2;
	}
}
