package org.processmining.blocklayout.models.reductions;

import org.processmining.blocklayout.models.blocks.Block;
import org.processmining.blocklayout.models.blocks.HorizontalBlock;
import org.processmining.blocklayout.models.statemachine.Place;
import org.processmining.blocklayout.models.statemachine.StateMachineNet;
import org.processmining.blocklayout.models.statemachine.Transition;
import org.processmining.blocklayout.parameters.LayoutParameters;

public class PlaceAbstractionReduction implements Reduction {

	/**
	 * The place involved in this reduction.
	 */
	private Place place;

	/**
	 * The transition involved in this reduction.
	 */
	private Transition transition;

	private int costs;

	/**
	 * Creates the reduction for the given place and the given transition.
	 * 
	 * @param place
	 *            The given place.
	 * @param transition
	 *            The given transition.
	 */
	public PlaceAbstractionReduction(Place place, Transition transition) {
		this.place = place;
		this.transition = transition;
		this.costs = -1;
	}

	/**
	 * Returns the costs of applying this reduction on the given net.
	 * 
	 * @param net
	 *            The given net
	 * @return The costs of applying this reduction on the given net.
	 */
	public int costs(StateMachineNet net, LayoutParameters layoutParameters) {
		if (costs == -1) {
			if (net.getPreset(net.getPostPlace(transition)).size() == 1) {
				costs = 0;
			} else {
				costs = layoutParameters.getPlaceAbstractionReductionCosts(net, place, transition);
			}
			System.out.println("[PlaceAbstractionReduction] Costs " + costs + " (" + place + ") + [" + transition
					+ "] -> (" + net.getPostPlace(transition) + ")");
		}
		return costs;
	}

	/**
	 * Applies this reduction to the given net.
	 * 
	 * @param net
	 *            The given net.
	 * @param layoutParameters
	 *            Parameters including, among others, the horizontal space
	 *            between element blocks.
	 */
	public void apply(StateMachineNet net, LayoutParameters layoutParameters) {
		System.out.println("[PlaceAbstractionReduction] " + costs(net, layoutParameters) + " (" + place + ") + ["
				+ transition + "] -> (" + net.getPostPlace(transition) + ")");
		Block block = new HorizontalBlock(place.getBlock(), transition.getBlock(),
				net.getPostPlace(transition).getBlock());
		net.getPostPlace(transition).setBlock(block);
		/*
		 * Remove the given place and given transition from the net.
		 */
		net.applyAbstraction(place, transition);
	}

}
