package org.processmining.blocklayout.models.reductions;

import java.util.HashSet;
import java.util.Set;

import org.processmining.blocklayout.models.blocks.Block;
import org.processmining.blocklayout.models.blocks.VerticalBlock;
import org.processmining.blocklayout.models.statemachine.Place;
import org.processmining.blocklayout.models.statemachine.StateMachineNet;
import org.processmining.blocklayout.models.statemachine.Transition;
import org.processmining.blocklayout.parameters.LayoutParameters;

public class PlaceReduction implements Reduction {

	private Place place;
	private Place proxy;
	private boolean isReverse;
	private int costs;

	public PlaceReduction(Place place, Place proxy, boolean isReverse) {
		this.place = place;
		this.proxy = proxy;
		this.isReverse = isReverse;
		this.costs = -1;
	}

	public int costs(StateMachineNet net, LayoutParameters layoutParameters) {
		if (costs == -1) {
			if (net.getPreset(place).size() == net.getPreset(proxy).size()
					&& net.getPostset(place).size() == net.getPostset(proxy).size()) {
				Set<Place> prePlace = new HashSet<Place>();
				Set<Place> preProxy = new HashSet<Place>();
				Set<Place> postPlace = new HashSet<Place>();
				Set<Place> postProxy = new HashSet<Place>();

				for (Transition transition : net.getPreset(place)) {
					prePlace.add(net.getPrePlace(transition));
				}
				for (Transition transition : net.getPreset(proxy)) {
					(isReverse ? postProxy : preProxy).add(net.getPrePlace(transition));
				}
				for (Transition transition : net.getPostset(place)) {
					postPlace.add(net.getPostPlace(transition));
				}
				for (Transition transition : net.getPostset(proxy)) {
					(isReverse ? preProxy : postProxy).add(net.getPostPlace(transition));
				}
				if (prePlace.equals(preProxy) && postPlace.equals(postProxy)) {
					// Place has same preset and postset as proxy. Ideal.
					costs = 0;
				} else if (preProxy.containsAll(prePlace) && postProxy.containsAll(postPlace)) {
					// Preset/postset of place contained in preset/postset of proxy. OK-ish.
					costs = 1;
				}
			}
			if (costs == -1) {
				costs = layoutParameters.getPlaceReductionCosts(net, place, proxy);
			}
			System.out.println(
					"[PlaceReduction] Costs " + isReverse + " " + costs + " (" + place + ") -> (" + proxy + ")");
		}
		return costs;
	}

	public void apply(StateMachineNet net, LayoutParameters layoutParameters) {
		System.out.println("[PlaceReduction] Assign " + isReverse + " " + costs(net, layoutParameters) + " (" + place
				+ ") -> (" + proxy + ")");
		for (Transition transition : net.getPreset(place)) {
			net.addArc(transition, proxy);
		}
		for (Transition transition : net.getPostset(place)) {
			net.addArc(proxy, transition);
		}
		Block block = new VerticalBlock(place.getBlock(), proxy.getBlock(), isReverse);
		proxy.setBlock(block);
		net.applyPlace(place, proxy);
	}

}
