package org.processmining.blocklayout.models.reductions;

import org.processmining.blocklayout.models.blocks.Block;
import org.processmining.blocklayout.models.blocks.DummyBlock;
import org.processmining.blocklayout.models.blocks.HorizontalBlock;
import org.processmining.blocklayout.models.statemachine.StateMachineNet;
import org.processmining.blocklayout.models.statemachine.Transition;
import org.processmining.blocklayout.parameters.LayoutParameters;

public class RigidReduction implements Reduction {

	/**
	 * This aims to remove a rigid. THe following three transitions are in a sequence,
	 * but two additional transitions exists: 
	 * 1. splitTransition + transition
	 * 2. transition + joinTransition
	 * As a result, this cannot be reduced using other rules without costs.
	 * To reduce this fragment, the input arc and output arc of transition are rerouted to
	 * the input place of splitTransition and the output place of joinTranstion.
	 * Two dummy blocks are used to keep the spacing of transition, if possible.
	 */
	private Transition splitTransition, transition, joinTransition;

	public RigidReduction(Transition splitTransition, Transition transition, Transition joinTransition) {
		this.splitTransition = splitTransition;
		this.transition = transition;
		this.joinTransition = joinTransition;
	}

	public int costs(StateMachineNet net, LayoutParameters layoutParameters) {
		return 0;
	}

	public void apply(StateMachineNet net, LayoutParameters layoutParameters) {
		System.out.println(
				"[RigidReduction] Assign (" + splitTransition + ", " + transition + ", " + joinTransition + ")");
		Block block = new HorizontalBlock(new DummyBlock(splitTransition.getBlock()), transition.getBlock(),
				new DummyBlock(joinTransition.getBlock()));
		transition.setBlock(block);
		net.applyRigid(splitTransition, transition, joinTransition);
	}

}
