package org.processmining.blocklayout.plugins;

import org.deckfour.uitopia.api.event.TaskListener.InteractionResult;
import org.processmining.acceptingpetrinet.models.AcceptingPetriNet;
import org.processmining.blocklayout.algorithms.LayoutAlgorithm;
import org.processmining.blocklayout.dialogs.LayoutDialog;
import org.processmining.blocklayout.parameters.LayoutParameters;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.connections.ConnectionCannotBeObtained;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.models.connections.GraphLayoutConnection;
import org.processmining.models.graphbased.directed.DirectedGraph;
import org.processmining.models.graphbased.directed.DirectedGraphEdge;
import org.processmining.models.graphbased.directed.DirectedGraphNode;
import org.processmining.models.graphbased.directed.bpmn.BPMNDiagram;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.graphbased.directed.transitionsystem.TransitionSystem;

public class LayoutPlugin extends LayoutAlgorithm {

	public final static String HELPPNUSER = "Applies the block layout to this Petri net. "
			+ "This layout replaces the JGraph layout, and is not used by GraphViz/Dot. "
			+ "Allows the user to select the parameters to be used for the block layout.";

	@Plugin(name = "Apply Block Layout to Petri Net, User Selects Parameters", returnLabels = {
			"Petri Net w/ Block Layout" }, returnTypes = {
					Petrinet.class }, parameterLabels = { "Petri Net" }, help = HELPPNUSER, userAccessible = true)
	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "H.M.W. Verbeek", email = "h.m.w.verbeek@tue.nl")
	@PluginVariant(requiredParameterLabels = { 0 })
	public Petrinet runUI(UIPluginContext context, Petrinet net) {
		LayoutParameters layoutParameters = new LayoutParameters();

		runUI(context, net, layoutParameters);
		return net;
	}

	public final static String HELPPNDEF = "Applies the block layout to this Petri net. "
			+ "This layout replaces the JGraph layout, and is not used by GraphViz/Dot. "
			+ "Uses default parameters for the block layout.";

	@Plugin(name = "Apply Block Layout to Petri Net, Default Parameters", returnLabels = {
			"Petri Net w/ Block Layout" }, returnTypes = {
					Petrinet.class }, parameterLabels = { "Petri Net" }, help = HELPPNDEF, userAccessible = true)
	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "H.M.W. Verbeek", email = "h.m.w.verbeek@tue.nl")
	@PluginVariant(requiredParameterLabels = { 0 })
	public Petrinet run(PluginContext context, Petrinet net) {
		LayoutParameters layoutParameters = new LayoutParameters();

		runDefault(context, net, layoutParameters);
		return net;
	}

	public final static String HELPAPNUSER = "Applies the block layout to this accepting Petri net. "
			+ "This layout replaces the JGraph layout, and is not used by GraphViz/Dot. "
			+ "Allows the user to select the parameters to be used for the block layout.";

	@Plugin(name = "Apply Block Layout to Accepting Petri Net, User Selects Parameters", returnLabels = {
			"Accepting Petri Net w/ Block Layout" }, returnTypes = { AcceptingPetriNet.class }, parameterLabels = {
					"Accepting Petri Net" }, help = HELPAPNUSER, userAccessible = true)
	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "H.M.W. Verbeek", email = "h.m.w.verbeek@tue.nl")
	@PluginVariant(requiredParameterLabels = { 0 })
	public AcceptingPetriNet runUI(UIPluginContext context, AcceptingPetriNet net) {
		LayoutParameters layoutParameters = new LayoutParameters();

		runUI(context, net.getNet(), layoutParameters);
		return net;
	}

	public final static String HELPAPNDEF = "Applies the block layout to this accepting Petri net. "
			+ "This layout replaces the JGraph layout, and is not used by GraphViz/Dot. "
			+ "Uses default parameters for the block layout.";

	@Plugin(name = "Apply Block Layout to Accepting Petri Net, Default Parameters", returnLabels = {
			"Accepting Petri Net w/ Block Layout" }, returnTypes = { AcceptingPetriNet.class }, parameterLabels = {
					"Accepting Petri Net" }, help = HELPAPNDEF, userAccessible = true)
	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "H.M.W. Verbeek", email = "h.m.w.verbeek@tue.nl")
	@PluginVariant(requiredParameterLabels = { 0 })
	public AcceptingPetriNet run(PluginContext context, AcceptingPetriNet net) {
		LayoutParameters layoutParameters = new LayoutParameters();

		runDefault(context, net.getNet(), layoutParameters);
		return net;
	}

	public final static String HELPBPMNUSER = "Applies the block layout to this BPMN diagram. "
			+ "This layout replaces the JGraph layout, and is not used by GraphViz/Dot. "
			+ "Allows the user to select the parameters to be used for the block layout.";

	@Plugin(name = "Apply Block Layout to BPMN Diagram, User Selects Parameters", returnLabels = {
			"BPMN Diagram w/ Block Layout" }, returnTypes = { BPMNDiagram.class }, parameterLabels = {
					"BPMN Diagram" }, help = HELPBPMNUSER, userAccessible = true)
	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "H.M.W. Verbeek", email = "h.m.w.verbeek@tue.nl")
	@PluginVariant(requiredParameterLabels = { 0 })
	public BPMNDiagram runUI(UIPluginContext context, BPMNDiagram diagram) {
		LayoutParameters layoutParameters = new LayoutParameters();

		runUI(context, diagram, layoutParameters);
		return diagram;
	}

	public final static String HELPBPMNDEF = "Applies the block layout to this BPMN diagram. "
			+ "This layout replaces the JGraph layout, and is not used by GraphViz/Dot. "
			+ "Uses default parameters for the block layout.";

	@Plugin(name = "Apply Block Layout to BPMN Diagram, Default Parameters", returnLabels = {
			"BPMN Diagram w/ Block Layout" }, returnTypes = { BPMNDiagram.class }, parameterLabels = {
					"BPMN Diagram" }, help = HELPBPMNDEF, userAccessible = true)
	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "H.M.W. Verbeek", email = "h.m.w.verbeek@tue.nl")
	@PluginVariant(requiredParameterLabels = { 0 })
	public BPMNDiagram run(PluginContext context, BPMNDiagram diagram) {
		LayoutParameters layoutParameters = new LayoutParameters();

		runDefault(context, diagram, layoutParameters);
		return diagram;
	}

	public final static String HELPTSUSER = "Applies the block layout to this transition system. "
			+ "This layout replaces the JGraph layout, and is not used by GraphViz/Dot. "
			+ "Allows the user to select the parameters to be used for the block layout.";

	@Plugin(name = "Apply Block Layout to Transition System, User Selects Parameters", returnLabels = {
			"Transition System w/ Block Layout" }, returnTypes = { TransitionSystem.class }, parameterLabels = {
					"Transition System" }, help = HELPTSUSER, userAccessible = true)
	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "H.M.W. Verbeek", email = "h.m.w.verbeek@tue.nl")
	@PluginVariant(requiredParameterLabels = { 0 })
	public TransitionSystem runUI(UIPluginContext context, TransitionSystem ts) {
		LayoutParameters layoutParameters = new LayoutParameters();
		layoutParameters.setLeftRight(false);

		runUI(context, ts, layoutParameters);
		return ts;
	}

	public final static String HELPTSDEF = "Applies the block layout to this transition system. "
			+ "This layout replaces the JGraph layout, and is not used by GraphViz/Dot. "
			+ "Uses default parameters for the block layout.";

	@Plugin(name = "Apply Block Layout to Transition System, Default Parameters", returnLabels = {
			"Transition System w/ Block Layout" }, returnTypes = { TransitionSystem.class }, parameterLabels = {
					"Transition System" }, help = HELPBPMNDEF, userAccessible = true)
	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "H.M.W. Verbeek", email = "h.m.w.verbeek@tue.nl")
	@PluginVariant(requiredParameterLabels = { 0 })
	public TransitionSystem run(PluginContext context, TransitionSystem ts) {
		LayoutParameters layoutParameters = new LayoutParameters();
		layoutParameters.setLeftRight(false);

		runDefault(context, ts, layoutParameters);
		return ts;
	}

	private void runUI(UIPluginContext context,
			DirectedGraph<? extends DirectedGraphNode, ? extends DirectedGraphEdge<? extends DirectedGraphNode, ? extends DirectedGraphNode>> graph,
			LayoutParameters layoutParameters) {
		LayoutDialog layoutDialog = new LayoutDialog(context, layoutParameters);
		InteractionResult result = context.showWizard("Configure layout parameters", true, true, layoutDialog);
		if (result != InteractionResult.FINISHED) {
			return;
		}
		runDefault(context, graph, layoutParameters);
	}

	private void runDefault(PluginContext context,
			DirectedGraph<? extends DirectedGraphNode, ? extends DirectedGraphEdge<? extends DirectedGraphNode, ? extends DirectedGraphNode>> graph,
			LayoutParameters layoutParameters) {
		GraphLayoutConnection layout;
		try {
			layout = context.getConnectionManager().getFirstConnection(GraphLayoutConnection.class, context, graph);
		} catch (ConnectionCannotBeObtained e) {
			layout = new GraphLayoutConnection(graph);
		}

		layout(graph, layout, layoutParameters);

		layout.setLayedOut(true);

		for (Petrinet net : layoutParameters.getNonZeroCostNets()) {
			context.getProvidedObjectManager().createProvidedObject(net.getLabel(), net, Petrinet.class, context);
		}
	}
}
