package org.processmining.blocklayout.plugins;

import org.processmining.acceptingpetrinet.models.AcceptingPetriNet;
import org.processmining.blocklayout.algorithms.StateMachineBuilderAlgorithm;
import org.processmining.blocklayout.models.statemachine.StateMachineNet;
import org.processmining.blocklayout.parameters.LayoutParameters;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.connections.ConnectionCannotBeObtained;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.models.connections.GraphLayoutConnection;
import org.processmining.models.graphbased.directed.DirectedGraph;
import org.processmining.models.graphbased.directed.DirectedGraphEdge;
import org.processmining.models.graphbased.directed.DirectedGraphNode;
import org.processmining.models.graphbased.directed.bpmn.BPMNDiagram;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.graphbased.directed.transitionsystem.TransitionSystem;

public class StateMachineBuilderPlugin extends StateMachineBuilderAlgorithm {

	public final static String HELPPNDEF = "Creates a state machine Petri net for Block Layout from the accepting Petri net.";

	@Plugin(name = "Create a state machine Petri net for Block Layout from the Petri net", returnLabels = {
			"State machine Petri net" }, returnTypes = {
					StateMachineNet.class }, parameterLabels = { "Petri Net" }, help = HELPPNDEF, userAccessible = true)
	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "H.M.W. Verbeek", email = "h.m.w.verbeek@tue.nl")
	@PluginVariant(requiredParameterLabels = { 0 })
	public StateMachineNet run(PluginContext context, Petrinet net) {
		return runDefault(context, net, new LayoutParameters());
	}

	public final static String HELPAPNDEF = "Creates a state machine Petri net for Block Layout from the accepting Petri net.";

	@Plugin(name = "Create a state machine Petri net for Block Layout from the accepting Petri net", returnLabels = {
			"State machine Petri net" }, returnTypes = { StateMachineNet.class }, parameterLabels = {
					"Accepting Petri Net" }, help = HELPAPNDEF, userAccessible = true)
	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "H.M.W. Verbeek", email = "h.m.w.verbeek@tue.nl")
	@PluginVariant(requiredParameterLabels = { 0 })
	public StateMachineNet run(PluginContext context, AcceptingPetriNet net) {
		return runDefault(context, net.getNet(), new LayoutParameters());
	}

	public final static String HELPBPMNDEF = "Creates a state machine Petri net for Block Layout from the BPMN diagram.";

	@Plugin(name = "Create a state machine Petri net for Block Layout from the BPMN diagram", returnLabels = {
			"State machine Petri net" }, returnTypes = { StateMachineNet.class }, parameterLabels = {
					"BPMN Diagram" }, help = HELPBPMNDEF, userAccessible = true)
	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "H.M.W. Verbeek", email = "h.m.w.verbeek@tue.nl")
	@PluginVariant(requiredParameterLabels = { 0 })
	public StateMachineNet run(PluginContext context, BPMNDiagram diagram) {
		return runDefault(context, diagram, new LayoutParameters());
	}

	public final static String HELPTSDEF = "Creates a state machine Petri net for Block Layout from the transition system.";

	@Plugin(name = "Create a state machine Petri net for Block Layout from the transition system", returnLabels = {
			"State machine Petri net" }, returnTypes = { StateMachineNet.class }, parameterLabels = {
					"Transition System" }, help = HELPTSDEF, userAccessible = true)
	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "H.M.W. Verbeek", email = "h.m.w.verbeek@tue.nl")
	@PluginVariant(requiredParameterLabels = { 0 })
	public StateMachineNet run(PluginContext context, TransitionSystem ts) {
		return runDefault(context, ts, new LayoutParameters());
	}

	private StateMachineNet runDefault(PluginContext context,
			DirectedGraph<? extends DirectedGraphNode, ? extends DirectedGraphEdge<? extends DirectedGraphNode, ? extends DirectedGraphNode>> graph,
			LayoutParameters layoutParameters) {
		GraphLayoutConnection layout;
		try {
			layout = context.getConnectionManager().getFirstConnection(GraphLayoutConnection.class, context, graph);
		} catch (ConnectionCannotBeObtained e) {
			layout = new GraphLayoutConnection(graph);
		}

		return build(graph, layout, layoutParameters).getFirst();
	}
}
